//
// Created by steven on 11/14/2024.
//

#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>

#include "../libs/go_p2fa.h"
#include "images.h"
#include "../libs/e-Paper/EPD_2in13_V4.h"
#include "../libs/GUI/GUI_Paint.h"
#include "../libs/GUI/GUI_BMPfile.h"

void Handler(int sigNum) {
    printf("Caught signal %d\n", sigNum);
    DEV_Module_Exit();
    exit(0);
}

int draw() {
    if (DEV_Module_Init() != 0) {
        return -1;
    }
    EPD_2in13_V4_Init();
    EPD_2in13_V4_Clear();

    UBYTE *BlackImage;
    UWORD ImageSize = ((EPD_2in13_V4_WIDTH % 8 == 0) ? (EPD_2in13_V4_WIDTH / 8) : (EPD_2in13_V4_WIDTH / 8 + 1)) * EPD_2in13_V4_HEIGHT;
    if ((BlackImage = (UBYTE *) malloc(ImageSize)) == NULL) {
        Debug("Failed to apply for black memory...\n");
        return -1;
    }
    Paint_NewImage(BlackImage, EPD_2in13_V4_WIDTH, EPD_2in13_V4_HEIGHT, 90, WHITE);
    Paint_Clear(WHITE);

    Paint_DrawString(32, 10, "Hello, World!", &Font16, WHITE, BLACK);

    const char *confName = "default";
    GoString confStr = {confName, strlen(confName)};
    const char *code = getCode(confStr);
    Debug("Code: %s\n", code);
    Paint_DrawString(5, 5, code, &Font24, WHITE, BLACK);

    Paint_DrawRectangle(10, 112, 240, 92, BLACK, DOT_PIXEL_1X1, DRAW_FILL_EMPTY);
    uint64_t timeLeft = getTimeRemainingMS(30);
    for (int j = 0; j < 3; ++j) {
        for (int i = getTimeRemainingMS(30); i > 200; i = getTimeRemainingMS(30)) {
            Paint_DrawRectangle(10, 112, (timeLeft / (30 * 1000)) * 240, 92, BLACK, DOT_PIXEL_1X1, DRAW_FILL_FULL);
            DEV_Delay_ms(100);
        }
    }

    DEV_Delay_ms(5000);

    Debug("Clear...\r\n");
    EPD_2in13_V4_Init();
    EPD_2in13_V4_Clear();

    Debug("Goto Sleep...\r\n");
    EPD_2in13_V4_Sleep();
    free(BlackImage);
    BlackImage = NULL;

    DEV_Delay_ms(2000);//important, at least 2s
    // close 5V
    Debug("close 5V, Module enters 0 power consumption ...\r\n");

    DEV_Module_Exit();
    return 0;
}

int main() {
    signal(SIGINT, Handler);

    loadConfigs();

    draw();

    return 0;
}